using System;
using System.IO;
using System.Net;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace WarmupSharePoint
{
	class Program
	{
		private static Version _Version = new Version(1,1,0,0);

		static void Main(string[] args)
		{
			try
			{
				Console.WriteLine("WarmupSharePoint ("+_Version+") by Ren Hzser");
				if (args.Length == 1 && args[0].Equals("Farm", StringComparison.InvariantCultureIgnoreCase))
				{
					Console.WriteLine("Working on the whole farm...");
					SPWebService contentService = SPWebService.ContentService;
					foreach(SPWebApplication app in contentService.WebApplications) 
					{
						foreach (SPSite site in app.Sites)
						{
							try
							 {
								 Console.WriteLine("Site {0}...", site.Url);
								 ReadSites(site.Url);
							 }
							 finally
							 {
								site.Dispose();
							 }
						}
					}
				}
				else if (args.Length == 1)
				{
					ReadSites(args[0]);
				}
				else if (args.Length == 2 && args[1].ToUpper() == "ALLSITES")
				{
					Console.WriteLine("Starting with {0}...", args[0]);
					ReadAllSites(args[0]);
					Console.WriteLine("...Finished {0}", args[0]);
				}
				else 
				{
					Console.WriteLine("Usage:");
					Console.WriteLine("WarmupSharePoint http://serverurl [AllSites]");
					Console.WriteLine("\tA start URL is required");
					Console.WriteLine("\t'AllSites' hit all Webs in all SiteCollections within the WebApplication");
					Console.WriteLine("WarmupSharePoint Farm");
					Console.WriteLine("\twill hil all Webs in the whole farm (except CentralAdministration)");
				}
			}
			catch (Exception ex)
			{
				Console.WriteLine(string.Format("Error in WarmupSharePoint:\r\n{0}", ex));
			}
		}

		private static void ReadAllSites(string url)
		{
			using (var site = new SPSite(url))
			{
				SPSiteCollection sites = site.WebApplication.Sites;
				foreach (SPSite subSite in sites)
				{
					try
					{
						using (SPWeb rootWeb = subSite.RootWeb)
						{
							Console.WriteLine(string.Format("Working on '{0}'", rootWeb.Url));
							ReadSites(rootWeb.Url);
						}
					}
					finally
					{
						subSite.Dispose();
					}
				}
			}
		}

		private static void ReadSites(string rootUrl)
		{
			try
			{
				using (var site = new SPSite(rootUrl))
				using (SPWeb rootWeb = site.RootWeb)
				{
					Console.Write("Hitting: {0} ", rootWeb.Title);
					Console.Write(CallWebsite(rootWeb.Url) + "\r\n");

					foreach (SPWeb web in rootWeb.GetSubwebsForCurrentUser())
					{
						try
						{
							ReadSubSites(web);
						}
						finally
						{
							web.Dispose();
						}
					}
				}
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.Message);
			}
		}

		private static void ReadSubSites(SPWeb web)
		{
			foreach (SPWeb subWeb in web.GetSubwebsForCurrentUser())
			{
				try
				{
					Console.Write("Hitting: {0} ", subWeb.Title);
					Console.Write(CallWebsite(subWeb.Url) + "\r\n");
					ReadSubSites(subWeb);
				}
				finally
				{
					subWeb.Dispose();
				}
			}
		}

		/// <summary>
		/// calls the Website by its URL and reads the Response to "warm it up"
		/// </summary>
		/// <param name="url">http://websiteurl</param>
		/// <returns>OK | ex.Message</returns>
		private static string CallWebsite(string url)
		{
			try
			{
				var sbuild = new StringBuilder();
				var req = (HttpWebRequest)WebRequest.Create(url);
				req.Credentials = CredentialCache.DefaultCredentials;

				using (var webresponse = (HttpWebResponse)req.GetResponse())
				using (var webstream = new StreamReader(webresponse.GetResponseStream(), Encoding.ASCII))
				{
					string temp;
					while ((temp = webstream.ReadLine()) != null)
					{
						sbuild.Append(temp + "\r\n");
					}
				}
			}
			catch (Exception ex)
			{
				return ex.Message;
			}
			return "OK";
		}
	}
}